/**************************************************************
Flash Magic Just In Time (JIT) Example
Serial Number/Datestamp Generation
Version 1.02 8051, ARM
Modified by Torben W. Hansen February 13th, 2009
**************************************************************/

// include needed header files
#include <stdio.h>
#include <time.h>
#include "hexfile.h"

#define ARM // compile for ARMs, forexample LC2378 - Remark this line for 8051s

// location in flash memory to store serial number at
#define SN_ADDR 0x0007D000     // Define 32bit address ARMs (LPC2378)   
//#define SN_ADDR 0xFFF0       // Define 16bit address 8051s 

// divide flash location in two 16bit parts
#define LBA  (SN_ADDR & 0xFFFF)// Define 16bit (low part) Linear Base Address
#define ULBA (SN_ADDR >> 16)   // Define 16bit (high part) Upper Linear Base Address

// function to generate the actual serial number and put it into
// an intel hex record ready for programming
unsigned long  generate_serialnumber(FILE *outputfile)
{
  unsigned char buffer[4];
  char hexrecord[100];
  int bytecount, recordtype;
  unsigned int address;
  unsigned long now;

  // get the current time 
  time((time_t *)&now); // Use this line for date/time serial number
  //now = 1234567890;   // Use this line for fixed serial number
  //now = 0x0F0F0F0F;   // Use this line for fixed serial number

#ifdef ARM // ARMs

  /////////////////////////////////////////////////////////////////////////////
  // create a Start Linear Address record.
  // Not necessary, but Keil compiler do so...
  /////////////////////////////////////////////////////////////////////////////
  bytecount  = 0x04;
  address    = 0x0000;
  recordtype = 0x05; // Start Linear Address record
  // Transfer Full 32 bit Start Address to buffer (High byte first) 
  buffer[0] = (SN_ADDR >> 24) & 0xFF;
  buffer[1] = (SN_ADDR >> 16) & 0xFF;
  buffer[2] = (SN_ADDR >> 8 ) & 0xFF;
  buffer[3] =  SN_ADDR & 0xFF;
  generate_hexrecord(bytecount, address, recordtype, buffer, hexrecord);
  // output the hex record
  fprintf(outputfile, "%s\n", hexrecord);

  /////////////////////////////////////////////////////////////////////////////
  // create an Upper Linear Base Address record 
  /////////////////////////////////////////////////////////////////////////////
  bytecount  = 0x02;
  address    = 0x0000;
  recordtype = 0x04; // Upper Linear Base Address record
  // Transfer Upper Linear Base Address to buffer (High byte first) 
  buffer[0] = (ULBA >> 8 ) & 0xFF;
  buffer[1] =  ULBA & 0xFF;
  generate_hexrecord(bytecount, address, recordtype, buffer, hexrecord);
  // output the hex record
  fprintf(outputfile, "%s\n", hexrecord);

  /////////////////////////////////////////////////////////////////////////////
  // create a Data record 
  /////////////////////////////////////////////////////////////////////////////
  bytecount  = 0x04;
  address    = LBA;
  recordtype = 0x00; // Data record
  // Transfer current time to buffer - little-endian for ARMs
  buffer[0] =  now & 0xFF;
  buffer[1] = (now >> 8 ) & 0xFF;
  buffer[2] = (now >> 16) & 0xFF;
  buffer[3] = (now >> 24) & 0xFF;

#else // 8051s

  /////////////////////////////////////////////////////////////////////////////
  // create a Data record 
  /////////////////////////////////////////////////////////////////////////////
  bytecount  = 0x04;
  address    = LBA;
  recordtype = 0x00; // Data record
  // Transfer current time to buffer - big-endian for 8051s
  buffer[0] = (now >> 24) & 0xFF;
  buffer[1] = (now >> 16) & 0xFF;
  buffer[2] = (now >> 8 ) & 0xFF;
  buffer[3] =  now & 0xFF;

#endif

  generate_hexrecord(bytecount, address, recordtype, buffer, hexrecord);
  // output the hex record
  fprintf(outputfile, "%s\n", hexrecord);
  //return serial number
  return now;
}
 
// function to generate some feedback for the user
// in this case we tell the user what the serial number generated
// is so it can be noted and maybe put on a sticker somewhere
void useroutput(FILE *outputfile, unsigned long sn)
{

  fprintf(outputfile, "Serial No: %10.10u - %s", sn, ctime(&sn));  // Decimal output
  //fprintf(outputfile, "Serial No: %8.8LX - %s", sn, ctime(&sn)); // Hex output
}

// function to remove carriage returns and newlines from the
// end of a line
void strip_newline(char *text)
{
  char *ptr = text;
  while (*ptr)
  {
    if (*ptr == '\n' || *ptr == '\r') *ptr = '\0';
    ptr++;
  }
}

 
// main function
int main(int argc, char **argv)
{
  FILE *commandfile;
  FILE *datafile;
  FILE *userfile;
  FILE *serialnumberfile; // Added by Torben W. Hansen 09.02.2009

  char datafilepath[2000];
  char userfilepath[2000];
  unsigned long sn;
  
  // if we didn't receive two arguments then we can't do anything
  if (argc < 2) return 0;

  // try to open the command file passed in the first argument
  commandfile = fopen(argv[1], "r");
  if (!commandfile) return 0;
  
  // try to read in the path to the file to create with hex records
  if (fgets(datafilepath, 2000, commandfile) == NULL)
  {
    fclose(commandfile);
    return 0;
  }

  // try to read in the path to the user output file to create
  if (fgets(userfilepath, 2000, commandfile) == NULL)
  {
    fclose(commandfile);
    return 0;
  }

  // we are done with the command file
  fclose(commandfile);

  // get rid of junk we don't need
  strip_newline(datafilepath);
  strip_newline(userfilepath);

  // create file for hex records and generate records
  datafile = fopen(datafilepath, "wb");
  if (!datafile) return 0;

  // generate the serial number and hex records. Return
  // serial number in order to write it into a text file.
  sn = generate_serialnumber(datafile);

  fclose(datafile);

  // create file for user output and put something in there
  userfile = fopen(userfilepath, "w");
  if (!userfile) return 0;
 
  useroutput(userfile, sn);

  fclose(userfile);

///////////////////////////////////////////////////////////////////////////////
//              Added by Torben W. Hansen - 09.02.2009
///////////////////////////////////////////////////////////////////////////////

  // write/append serial number information to text file
  serialnumberfile = fopen("serialnumbers.txt", "a");
  if (!serialnumberfile) return 0;
  useroutput(serialnumberfile, sn);
  fclose(serialnumberfile);
///////////////////////////////////////////////////////////////////////////////

  // done

  return 1;
}
